
local LII = LibStub:GetLibrary("LibItemInfo-1.0")

-------------------------------------------------------------------------------------------------
--  Colors  --
-------------------------------------------------------------------------------------------------
local colorYellow 		= "|cFFFF00" 	-- yellow 
local colorLightYellow 	= "|cFFFFCC" 	-- light yellow 
local colorGreen	 	= "|c996633" 	-- colorGreen
local colorGreen2	 	= "|cFFFF00" 	-- colorGreen
local colorMagenta		= "|cFF00FF"	-- Magenta
local colorRed 			= "|cFF0000" 	-- Red
local colorDrkOrange 	= "|cFFA500"	-- Dark Orange
local iconYellow		= "|cFFFF33"	-- icon yellow converted to hex: 1, 1, .2
local iconOrange		= "|cFF6600"	-- icon orange converted to hex: 1, .4, 0

function JunkIt.PrintApplyJunkFiltersMsg(_iNumItemsJunked, _iNumItemsUnJunked)
	d(colorRed.."JunkIt: "..colorMagenta.."Number of items junked: ".._iNumItemsJunked)
	d(colorRed.."JunkIt: "..colorDrkOrange.."Number of items unjunked: ".._iNumItemsUnJunked)
	
end
function JunkIt.AddDebugMsg(_bIsJunk, _lLink, _sMsg)
	local debugWin = JunkIt.DebugWin
	
	if _bKeep then
		JunkIt.DebugWin:AddText(colorGreen.."Keeping ".._lLink..": "..colorYellow.." ".._sMsg)
		return
	end
	JunkIt.DebugWin:AddText(colorRed.."Junking ".._lLink..": "..colorYellow.." ".._sMsg)
end

function JunkIt.PrintGoldMessage(_Reason, _bGain)
	if JunkIt.SavedVariables["GOLDMESSAGES"] then
		if _bGain then
			d(colorRed.."JunkIt: "..colorYellow.."You gained: "..zo_strformat(SI_MONEY_FORMAT, JunkIt.GoldGainHold)..".")
		else
			d(colorRed.."JunkIt: "..colorYellow.."You spent: "..zo_strformat(SI_MONEY_FORMAT, math.abs(JunkIt.GoldLostHold))..".")
		
		end
	end
end

function JunkIt.PrintRepairMessage(_bAllItems, _iRepairCost)
	if (JunkIt.SavedVariables["REPAIRMESSAGES"] and (_iRepairCost > 0)) then
		if _bAllItems then
			d(colorRed.."JunkIt: "..colorYellow.."All Items Repaired, cost: ".._iRepairCost.." gold.")
		else
			d(colorRed.."JunkIt: "..colorYellow.."Equipped Items Repaired, cost: ".._iRepairCost.." gold.")
		end
	end
end
--------------------------------------------------------------------------------------
--  Function for printing chat msg when Recipes/Researchable items are found --
--------------------------------------------------------------------------------------
function JunkIt.PrintChatMessages(_BagIdOrLink,  _iSlotId, _bIsJunk)
	local lItemLink = LII:GetItemLink(_BagIdOrLink, _iSlotId)
	local iItemType = GetItemLinkItemType(lItemLink)

	
	-- Looted Junk / Not Junk Item --
	if (JunkIt.SavedVariables["NOTJUNKEDMESSAGE"] and (not _bIsJunk)) then
		d(colorRed.."JunkIt: "..colorDrkOrange.."Looted Item: "..zo_strformat(SI_TOOLTIP_ITEM_NAME, lItemLink))
	elseif (JunkIt.SavedVariables["JUNKEDMESSAGE"] and _bIsJunk) then
		d(colorRed.."JunkIt: "..colorMagenta.."Looted Junk: "..zo_strformat(SI_TOOLTIP_ITEM_NAME, lItemLink))
	end
	
	----------------------------------------
	-- Unknown Research / Recipe Messages --
	----------------------------------------
	local tNeedInfo = JunkIt.KeepForResearch(_BagIdOrLink, _iSlotId)
	
	if not tNeedInfo then return end
	local bAreOthersWhoNeedOnWatchList = JunkIt.AreOtherCharsOnWatchlist(tNeedInfo)
	
	local bShowPlayerResearchMsgs = JunkIt.SavedVariables["UNKNOWNRESEARCHMESSAGESPLAYER"]
	local bShowOtherResearchMsgs = JunkIt.SavedVariables["UNKNOWNRESEARCHMESSAGESOTHER"]
	
	if bShowPlayerResearchMsgs and tNeedInfo.PlayerNeeds then
		if iItemType == ITEMTYPE_RECIPE then
			d(colorRed.."JunkIt: "..iconOrange.."Unknown recipe found: "..zo_strformat(SI_TOOLTIP_ITEM_NAME, lItemLink))
		else
			d(colorRed.."JunkIt: "..iconOrange.."Research item found: "..zo_strformat(SI_TOOLTIP_ITEM_NAME, lItemLink))
		end
	elseif bShowOtherResearchMsgs and bAreOthersWhoNeedOnWatchList then
		if iItemType == ITEMTYPE_RECIPE then
			d(colorRed.."JunkIt: "..iconYellow.."Unknown recipe found: "..zo_strformat(SI_TOOLTIP_ITEM_NAME, lItemLink))
		else
			d(colorRed.."JunkIt: "..iconYellow.."Research item found: "..zo_strformat(SI_TOOLTIP_ITEM_NAME, lItemLink))
		end
	end
	
	--only print if player needs or others who need are on watch list
	--if not (bAreOthersWhoNeedOnWatchList or tNeedInfo.PlayerNeeds) then return end
	local sCharsWhoNeedTrait = JunkIt.GetCharNamesStringFromTable(tNeedInfo)
		
	-- Characters Who Need Message --
	if (JunkIt.SavedVariables["UNKNOWNRESEARCHMESSAGESNAMES"] and (sCharsWhoNeedTrait ~= "")) then
		d(colorRed.."JunkIt: "..colorYellow.."Item needed by: "..sCharsWhoNeedTrait)
	end
end





